@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -56,8 +56,6 @@ static const char *type2string(enum connman_element_type type)
 		return "network";
 	case CONNMAN_ELEMENT_TYPE_SERVICE:
 		return "service";
-	case CONNMAN_ELEMENT_TYPE_PPP:
-		return "ppp";
 	case CONNMAN_ELEMENT_TYPE_IPV4:
 		return "ipv4";
 	case CONNMAN_ELEMENT_TYPE_IPV6:
@@ -77,27 +75,6 @@ static const char *type2string(enum connman_element_type type)
 	return NULL;
 }
 
-static void emit_element_signal(DBusConnection *conn, const char *member,
-					struct connman_element *element)
-{
-	DBusMessage *signal;
-
-	if (__connman_debug_enabled() == FALSE)
-		return;
-
-	DBG("conn %p member %s", conn, member);
-
-	if (element == NULL)
-		return;
-
-	signal = dbus_message_new_signal(element->path,
-					CONNMAN_DEBUG_INTERFACE, member);
-	if (signal == NULL)
-		return;
-
-	g_dbus_send_message(conn, signal);
-}
-
 struct foreach_data {
 	enum connman_element_type type;
 	element_cb_t callback;
@@ -199,52 +176,6 @@ void __connman_element_list(struct connman_element *element,
 						append_path, &filter);
 }
 
-struct count_data {
-	enum connman_element_type type;
-	int count;
-};
-
-static gboolean count_element(GNode *node, gpointer user_data)
-{
-	struct connman_element *element = node->data;
-	struct count_data *data = user_data;
-
-	DBG("element %p name %s", element, element->name);
-
-	if (element->type == CONNMAN_ELEMENT_TYPE_ROOT)
-		return FALSE;
-
-	if (data->type != CONNMAN_ELEMENT_TYPE_UNKNOWN &&
-					data->type != element->type)
-		return FALSE;
-
-	data->count++;
-
-	return FALSE;
-}
-
-int __connman_element_count(struct connman_element *element,
-					enum connman_element_type type)
-{
-	struct count_data data = { type, 0 };
-	GNode *node;
-
-	DBG("");
-
-	if (element != NULL) {
-		node = g_node_find(element_root, G_PRE_ORDER,
-						G_TRAVERSE_ALL, element);
-		if (node == NULL)
-			return 0;
-	} else
-		node = element_root;
-
-	g_node_traverse(node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
-						count_element, &data);
-
-	return data.count;
-}
-
 static struct connman_network *__connman_element_get_network(struct connman_element *element)
 {
 	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK &&
@@ -274,16 +205,12 @@ struct connman_service *__connman_element_get_service(struct connman_element *el
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
 	case CONNMAN_DEVICE_TYPE_VENDOR:
 	case CONNMAN_DEVICE_TYPE_GPS:
-	case CONNMAN_DEVICE_TYPE_NOZOMI:
-	case CONNMAN_DEVICE_TYPE_HUAWEI:
-	case CONNMAN_DEVICE_TYPE_NOVATEL:
 		break;
 	case CONNMAN_DEVICE_TYPE_ETHERNET:
-		service = __connman_service_lookup_from_device(device);
-		break;
 	case CONNMAN_DEVICE_TYPE_WIFI:
 	case CONNMAN_DEVICE_TYPE_WIMAX:
 	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
 	case CONNMAN_DEVICE_TYPE_MBM:
 	case CONNMAN_DEVICE_TYPE_HSO:
 		network = __connman_element_get_network(element);
@@ -381,9 +308,12 @@ static gboolean request_scan(GNode *node, gpointer user_data)
 
 	switch (type) {
 	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_SYSTEM:
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
 	case CONNMAN_SERVICE_TYPE_CELLULAR:
+	case CONNMAN_SERVICE_TYPE_GPS:
+	case CONNMAN_SERVICE_TYPE_VPN:
 		return FALSE;
 	case CONNMAN_SERVICE_TYPE_WIFI:
 	case CONNMAN_SERVICE_TYPE_WIMAX:
@@ -424,6 +354,9 @@ static gboolean enable_technology(GNode *node, gpointer user_data)
 
 	switch (type) {
 	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_SYSTEM:
+	case CONNMAN_SERVICE_TYPE_GPS:
+	case CONNMAN_SERVICE_TYPE_VPN:
 		return FALSE;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 	case CONNMAN_SERVICE_TYPE_WIFI:
@@ -467,6 +400,9 @@ static gboolean disable_technology(GNode *node, gpointer user_data)
 
 	switch (type) {
 	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_SYSTEM:
+	case CONNMAN_SERVICE_TYPE_GPS:
+	case CONNMAN_SERVICE_TYPE_VPN:
 		return FALSE;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 	case CONNMAN_SERVICE_TYPE_WIFI:
@@ -526,9 +462,7 @@ static gboolean probe_driver(GNode *node, gpointer data)
 		if (driver->probe(element) < 0)
 			return FALSE;
 
-		__connman_element_lock(element);
 		element->driver = driver;
-		__connman_element_unlock(element);
 	}
 
 	return FALSE;
@@ -588,9 +522,7 @@ static gboolean remove_driver(GNode *node, gpointer data)
 		if (driver->remove)
 			driver->remove(element);
 
-		__connman_element_lock(element);
 		element->driver = NULL;
-		__connman_element_unlock(element);
 	}
 
 	return FALSE;
@@ -623,6 +555,15 @@ static void unregister_property(gpointer data)
 	g_free(property);
 }
 
+static void unregister_child(gpointer data)
+{
+	struct connman_element *element = data;
+
+	DBG("element %p", element);
+
+	connman_element_unref(element);
+}
+
 void __connman_element_initialize(struct connman_element *element)
 {
 	DBG("element %p", element);
@@ -636,7 +577,8 @@ void __connman_element_initialize(struct connman_element *element)
 	element->index   = -1;
 	element->enabled = FALSE;
 
-	element->configuring = FALSE;
+	element->children = g_hash_table_new_full(g_str_hash, g_str_equal,
+						NULL, unregister_child);
 
 	element->properties = g_hash_table_new_full(g_str_hash, g_str_equal,
 						g_free, unregister_property);
@@ -680,12 +622,16 @@ static void free_properties(struct connman_element *element)
 {
 	DBG("element %p name %s", element, element->name);
 
-	__connman_element_lock(element);
-
 	g_hash_table_destroy(element->properties);
 	element->properties = NULL;
+}
 
-	__connman_element_unlock(element);
+static void free_children(struct connman_element *element)
+{
+	DBG("element %p name %s", element, element->name);
+
+	g_hash_table_destroy(element->children);
+	element->children = NULL;
 }
 
 void connman_element_unref(struct connman_element *element)
@@ -696,6 +642,7 @@ void connman_element_unref(struct connman_element *element)
 	if (g_atomic_int_dec_and_test(&element->refcount) == TRUE) {
 		if (element->destruct)
 			element->destruct(element);
+		free_children(element);
 		free_properties(element);
 		g_free(element->ipv4.address);
 		g_free(element->ipv4.netmask);
@@ -703,6 +650,8 @@ void connman_element_unref(struct connman_element *element)
 		g_free(element->ipv4.network);
 		g_free(element->ipv4.broadcast);
 		g_free(element->ipv4.nameserver);
+		g_free(element->ipv4.timeserver);
+		g_free(element->ipv4.pac);
 		g_free(element->devname);
 		g_free(element->path);
 		g_free(element->name);
@@ -741,12 +690,8 @@ static int set_static_property(struct connman_element *element,
 		break;
 	}
 
-	__connman_element_lock(element);
-
 	g_hash_table_replace(element->properties, g_strdup(name), property);
 
-	__connman_element_unlock(element);
-
 	return 0;
 }
 
@@ -781,58 +726,11 @@ static int set_static_array_property(struct connman_element *element,
 		break;
 	}
 
-	__connman_element_lock(element);
-
 	g_hash_table_replace(element->properties, g_strdup(name), property);
 
-	__connman_element_unlock(element);
-
 	return 0;
 }
 
-#if 0
-static int set_property(struct connman_element *element,
-				enum connman_property_id id, const void *value)
-{
-	switch (id) {
-	case CONNMAN_PROPERTY_ID_IPV4_ADDRESS:
-		__connman_element_lock(element);
-		g_free(element->ipv4.address);
-		element->ipv4.address = g_strdup(*((const char **) value));
-		__connman_element_unlock(element);
-		break;
-	case CONNMAN_PROPERTY_ID_IPV4_NETMASK:
-		__connman_element_lock(element);
-		g_free(element->ipv4.netmask);
-		element->ipv4.netmask = g_strdup(*((const char **) value));
-		__connman_element_unlock(element);
-		break;
-	case CONNMAN_PROPERTY_ID_IPV4_GATEWAY:
-		__connman_element_lock(element);
-		g_free(element->ipv4.gateway);
-		element->ipv4.gateway = g_strdup(*((const char **) value));
-		__connman_element_unlock(element);
-		break;
-	case CONNMAN_PROPERTY_ID_IPV4_BROADCAST:
-		__connman_element_lock(element);
-		g_free(element->ipv4.broadcast);
-		element->ipv4.broadcast = g_strdup(*((const char **) value));
-		__connman_element_unlock(element);
-		break;
-	case CONNMAN_PROPERTY_ID_IPV4_NAMESERVER:
-		__connman_element_lock(element);
-		g_free(element->ipv4.nameserver);
-		element->ipv4.nameserver = g_strdup(*((const char **) value));
-		__connman_element_unlock(element);
-		break;
-	default:
-		return -EINVAL;
-	}
-
-	return 0;
-}
-#endif
-
 int connman_element_get_value(struct connman_element *element,
 				enum connman_property_id id, void *value)
 {
@@ -844,49 +742,49 @@ int connman_element_get_value(struct connman_element *element,
 		if (element->ipv4.method == CONNMAN_IPCONFIG_METHOD_UNKNOWN)
 			return connman_element_get_value(element->parent,
 								id, value);
-		__connman_element_lock(element);
 		*((const char **) value) = __connman_ipconfig_method2string(element->ipv4.method);
-		__connman_element_unlock(element);
 		break;
 	case CONNMAN_PROPERTY_ID_IPV4_ADDRESS:
 		if (element->ipv4.address == NULL)
 			return connman_element_get_value(element->parent,
 								id, value);
-		__connman_element_lock(element);
 		*((char **) value) = element->ipv4.address;
-		__connman_element_unlock(element);
 		break;
 	case CONNMAN_PROPERTY_ID_IPV4_NETMASK:
 		if (element->ipv4.netmask == NULL)
 			return connman_element_get_value(element->parent,
 								id, value);
-		__connman_element_lock(element);
 		*((char **) value) = element->ipv4.netmask;
-		__connman_element_unlock(element);
 		break;
 	case CONNMAN_PROPERTY_ID_IPV4_GATEWAY:
 		if (element->ipv4.gateway == NULL)
 			return connman_element_get_value(element->parent,
 								id, value);
-		__connman_element_lock(element);
 		*((char **) value) = element->ipv4.gateway;
-		__connman_element_unlock(element);
 		break;
 	case CONNMAN_PROPERTY_ID_IPV4_BROADCAST:
 		if (element->ipv4.broadcast == NULL)
 			return connman_element_get_value(element->parent,
 								id, value);
-		__connman_element_lock(element);
 		*((char **) value) = element->ipv4.broadcast;
-		__connman_element_unlock(element);
 		break;
 	case CONNMAN_PROPERTY_ID_IPV4_NAMESERVER:
 		if (element->ipv4.nameserver == NULL)
 			return connman_element_get_value(element->parent,
 								id, value);
-		__connman_element_lock(element);
 		*((char **) value) = element->ipv4.nameserver;
-		__connman_element_unlock(element);
+		break;
+	case CONNMAN_PROPERTY_ID_IPV4_TIMESERVER:
+		if (element->ipv4.timeserver == NULL)
+			return connman_element_get_value(element->parent,
+								id, value);
+		*((char **) value) = element->ipv4.timeserver;
+		break;
+	case CONNMAN_PROPERTY_ID_IPV4_PAC:
+		if (element->ipv4.pac == NULL)
+			return connman_element_get_value(element->parent,
+								id, value);
+		*((char **) value) = element->ipv4.pac;
 		break;
 	default:
 		return -EINVAL;
@@ -903,8 +801,6 @@ static gboolean get_static_property(struct connman_element *element,
 
 	DBG("element %p name %s", element, element->name);
 
-	__connman_element_lock(element);
-
 	property = g_hash_table_lookup(element->properties, name);
 	if (property != NULL) {
 		switch (property->type) {
@@ -920,8 +816,6 @@ static gboolean get_static_property(struct connman_element *element,
 		}
 	}
 
-	__connman_element_unlock(element);
-
 	if (found == FALSE && element->parent != NULL)
 		return get_static_property(element->parent, name, value);
 
@@ -936,8 +830,6 @@ static gboolean get_static_array_property(struct connman_element *element,
 
 	DBG("element %p name %s", element, element->name);
 
-	__connman_element_lock(element);
-
 	property = g_hash_table_lookup(element->properties, name);
 	if (property != NULL) {
 		*((void **) value) = property->value;
@@ -945,35 +837,9 @@ static gboolean get_static_array_property(struct connman_element *element,
 		found = TRUE;
 	}
 
-	__connman_element_unlock(element);
-
 	return found;
 }
 
-#if 0
-static gboolean match_static_property(struct connman_element *element,
-					const char *name, const void *value)
-{
-	struct connman_property *property;
-	gboolean result = FALSE;
-
-	DBG("element %p name %s", element, element->name);
-
-	__connman_element_lock(element);
-
-	property = g_hash_table_lookup(element->properties, name);
-	if (property != NULL) {
-		if (property->type == DBUS_TYPE_STRING)
-			result = g_str_equal(property->value,
-						*((const char **) value));
-	}
-
-	__connman_element_unlock(element);
-
-	return result;
-}
-#endif
-
 /**
  * connman_element_set_string:
  * @element: element structure
@@ -1105,186 +971,6 @@ const void *connman_element_get_blob(struct connman_element *element,
 	return value;
 }
 
-int __connman_element_append_ipv4(struct connman_element *element,
-						DBusMessageIter *dict)
-{
-	const char *method = NULL;
-	const char *address = NULL, *netmask = NULL, *gateway = NULL;
-	const char *broadcast = NULL, *nameserver = NULL;
-
-	connman_element_get_value(element,
-				CONNMAN_PROPERTY_ID_IPV4_METHOD, &method);
-
-	connman_element_get_value(element,
-				CONNMAN_PROPERTY_ID_IPV4_ADDRESS, &address);
-	connman_element_get_value(element,
-				CONNMAN_PROPERTY_ID_IPV4_NETMASK, &netmask);
-	connman_element_get_value(element,
-				CONNMAN_PROPERTY_ID_IPV4_GATEWAY, &gateway);
-	connman_element_get_value(element,
-			CONNMAN_PROPERTY_ID_IPV4_BROADCAST, &broadcast);
-	connman_element_get_value(element,
-			CONNMAN_PROPERTY_ID_IPV4_NAMESERVER, &nameserver);
-
-	if (method != NULL)
-		connman_dbus_dict_append_variant(dict, "IPv4.Method",
-						DBUS_TYPE_STRING, &method);
-
-	if (address != NULL)
-		connman_dbus_dict_append_variant(dict, "IPv4.Address",
-						DBUS_TYPE_STRING, &address);
-
-	if (netmask != NULL)
-		connman_dbus_dict_append_variant(dict, "IPv4.Netmask",
-						DBUS_TYPE_STRING, &netmask);
-
-	if (gateway != NULL)
-		connman_dbus_dict_append_variant(dict, "IPv4.Gateway",
-						DBUS_TYPE_STRING, &gateway);
-
-	if (broadcast != NULL)
-		connman_dbus_dict_append_variant(dict, "IPv4.Broadcast",
-						DBUS_TYPE_STRING, &broadcast);
-
-	if (nameserver != NULL)
-		connman_dbus_dict_append_variant(dict, "IPv4.Nameserver",
-						DBUS_TYPE_STRING, &nameserver);
-
-	return 0;
-}
-
-int __connman_element_set_ipv4(struct connman_element *element,
-				const char *name, DBusMessageIter *value)
-{
-	int type;
-
-	type = dbus_message_iter_get_arg_type(value);
-
-	if (g_str_equal(name, "IPv4.Method") == TRUE) {
-		enum connman_ipconfig_method method;
-		const char *str;
-
-		if (type != DBUS_TYPE_STRING)
-			return -EINVAL;
-
-		dbus_message_iter_get_basic(value, &str);
-		method = __connman_ipconfig_string2method(str);
-		if (method == CONNMAN_IPCONFIG_METHOD_UNKNOWN)
-			return -EINVAL;
-
-		if (method == element->ipv4.method)
-			return -EALREADY;
-
-		element->ipv4.method = method;
-
-		connman_element_update(element);
-	} else if (g_str_equal(name, "IPv4.Address") == TRUE) {
-		const char *address;
-
-		if (type != DBUS_TYPE_STRING)
-			return -EINVAL;
-
-		dbus_message_iter_get_basic(value, &address);
-
-		g_free(element->ipv4.address);
-		element->ipv4.address = g_strdup(address);
-
-		connman_element_update(element);
-	} else if (g_str_equal(name, "IPv4.Netmask") == TRUE) {
-		const char *netmask;
-
-		if (type != DBUS_TYPE_STRING)
-			return -EINVAL;
-
-		dbus_message_iter_get_basic(value, &netmask);
-
-		g_free(element->ipv4.netmask);
-		element->ipv4.netmask = g_strdup(netmask);
-
-		connman_element_update(element);
-	} else if (g_str_equal(name, "IPv4.Gateway") == TRUE) {
-		const char *gateway;
-
-		if (type != DBUS_TYPE_STRING)
-			return -EINVAL;
-
-		dbus_message_iter_get_basic(value, &gateway);
-
-		g_free(element->ipv4.gateway);
-		element->ipv4.gateway = g_strdup(gateway);
-
-		connman_element_update(element);
-	} else if (g_str_equal(name, "IPv4.Broadcast") == TRUE) {
-		const char *broadcast;
-
-		if (type != DBUS_TYPE_STRING)
-			return -EINVAL;
-
-		dbus_message_iter_get_basic(value, &broadcast);
-
-		g_free(element->ipv4.broadcast);
-		element->ipv4.broadcast = g_strdup(broadcast);
-
-		connman_element_update(element);
-	} else if (g_str_equal(name, "IPv4.Nameserver") == TRUE) {
-		const char *nameserver;
-
-		if (type != DBUS_TYPE_STRING)
-			return -EINVAL;
-
-		dbus_message_iter_get_basic(value, &nameserver);
-
-		g_free(element->ipv4.nameserver);
-		element->ipv4.nameserver = g_strdup(nameserver);
-
-		connman_element_update(element);
-	}
-
-	return 0;
-}
-
-static void append_state(DBusMessageIter *entry, const char *state)
-{
-	DBusMessageIter value;
-	const char *key = "State";
-
-	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_STRING_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_STRING, &state);
-	dbus_message_iter_close_container(entry, &value);
-}
-
-static void emit_state_change(DBusConnection *conn, const char *state)
-{
-	DBusMessage *signal;
-	DBusMessageIter entry;
-
-	DBG("conn %p", conn);
-
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	append_state(&entry, state);
-
-	g_dbus_send_message(conn, signal);
-
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "StateChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &state);
-
-	g_dbus_send_message(conn, signal);
-}
-
 static void probe_element(struct connman_element *element)
 {
 	GSList *list;
@@ -1300,9 +986,7 @@ static void probe_element(struct connman_element *element)
 		DBG("driver %p name %s", driver, driver->name);
 
 		if (driver->probe(element) == 0) {
-			__connman_element_lock(element);
 			element->driver = driver;
-			__connman_element_unlock(element);
 			break;
 		}
 	}
@@ -1314,8 +998,6 @@ static void register_element(gpointer data, gpointer user_data)
 	const gchar *basepath;
 	GNode *node;
 
-	__connman_element_lock(element);
-
 	if (element->parent) {
 		node = g_node_find(element_root, G_PRE_ORDER,
 					G_TRAVERSE_ALL, element->parent);
@@ -1324,13 +1006,11 @@ static void register_element(gpointer data, gpointer user_data)
 		element->parent = element_root->data;
 
 		node = element_root;
-		basepath = "/device";
+		basepath = CONNMAN_PATH "/device";
 	}
 
 	element->path = g_strdup_printf("%s/%s", basepath, element->name);
 
-	__connman_element_unlock(element);
-
 	if (node == NULL) {
 		connman_error("Element registration for %s failed",
 							element->path);
@@ -1341,31 +1021,6 @@ static void register_element(gpointer data, gpointer user_data)
 
 	g_node_append_data(node, element);
 
-	if (element->type == CONNMAN_ELEMENT_TYPE_DHCP) {
-		element->parent->configuring = TRUE;
-
-#if 0
-		if (__connman_element_count(NULL,
-					CONNMAN_ELEMENT_TYPE_CONNECTION) == 0)
-			emit_state_change(connection, "connecting");
-#endif
-	}
-
-	if (element->type == CONNMAN_ELEMENT_TYPE_CONNECTION) {
-		struct connman_element *parent = element->parent;
-
-		while (parent) {
-			parent->configuring = FALSE;
-			parent = parent->parent;
-		}
-
-		if (__connman_element_count(NULL,
-					CONNMAN_ELEMENT_TYPE_CONNECTION) == 1)
-			emit_state_change(connection, "online");
-	}
-
-	emit_element_signal(connection, "ElementAdded", element);
-
 	if (started == FALSE)
 		return;
 
@@ -1412,7 +1067,7 @@ int connman_element_register(struct connman_element *element,
 	if (element->devname == NULL)
 		element->devname = g_strdup(element->name);
 
-        if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
+	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
 		goto setup;
 
 	if (__connman_element_device_isfiltered(element->devname) == TRUE)
@@ -1422,12 +1077,9 @@ setup:
 	if (connman_element_ref(element) == NULL)
 		return -EINVAL;
 
-	__connman_element_lock(element);
-
 	if (element->name == NULL) {
 		element->name = g_strdup(type2string(element->type));
 		if (element->name == NULL) {
-			__connman_element_unlock(element);
 			return -EINVAL;
 		}
 	}
@@ -1437,8 +1089,6 @@ setup:
 
 	element->parent = parent;
 
-	__connman_element_unlock(element);
-
 	register_element(element, NULL);
 
 	return 0;
@@ -1461,22 +1111,12 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 		if (element->driver->remove)
 			element->driver->remove(element);
 
-		__connman_element_lock(element);
 		element->driver = NULL;
-		__connman_element_unlock(element);
 	}
 
 	if (node != NULL)
 		g_node_destroy(node);
 
-	if (element->type == CONNMAN_ELEMENT_TYPE_CONNECTION) {
-		if (__connman_element_count(NULL,
-					CONNMAN_ELEMENT_TYPE_CONNECTION) == 0)
-			emit_state_change(connection, "offline");
-	}
-
-	emit_element_signal(connection, "ElementRemoved", element);
-
 	connman_element_unref(element);
 
 	return FALSE;
@@ -1517,8 +1157,6 @@ static gboolean update_element(GNode *node, gpointer user_data)
 	if (element->driver && element->driver->update)
 		element->driver->update(element);
 
-	emit_element_signal(connection, "ElementUpdated", element);
-
 	return FALSE;
 }
 
@@ -1610,8 +1248,11 @@ int __connman_element_init(const char *device, const char *nodevice)
 
 	element_root = g_node_new(element);
 
+	__connman_technology_init();
 	__connman_notifier_init();
+	__connman_location_init();
 	__connman_service_init();
+	__connman_provider_init();
 	__connman_network_init();
 	__connman_device_init();
 
@@ -1651,6 +1292,7 @@ void __connman_element_start(void)
 
 	__connman_connection_init();
 	__connman_ipv4_init();
+	__connman_dhcp_init();
 
 	if (__connman_rfkill_init() < 0)
 		__connman_udev_enable_rfkill_processing();
@@ -1662,7 +1304,9 @@ void __connman_element_stop(void)
 
 	__connman_rfkill_cleanup();
 
+	__connman_dhcp_cleanup();
 	__connman_ipv4_cleanup();
+	__connman_provider_cleanup();
 	__connman_connection_cleanup();
 }
 
@@ -1676,9 +1320,7 @@ static gboolean free_driver(GNode *node, gpointer data)
 		if (element->driver->remove)
 			element->driver->remove(element);
 
-		__connman_element_lock(element);
 		element->driver = NULL;
-		__connman_element_unlock(element);
 	}
 
 	return FALSE;
@@ -1703,7 +1345,9 @@ void __connman_element_cleanup(void)
 	__connman_device_cleanup();
 	__connman_network_cleanup();
 	__connman_service_cleanup();
+	__connman_location_cleanup();
 	__connman_notifier_cleanup();
+	__connman_technology_cleanup();
 
 	g_node_traverse(element_root, G_POST_ORDER, G_TRAVERSE_ALL, -1,
 							free_driver, NULL);
