@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -60,15 +60,10 @@ static DBusMessage *get_properties(DBusConnection *conn,
 			DBUS_TYPE_OBJECT_PATH, __connman_service_list, NULL);
 	connman_dbus_dict_append_array(&dict, "Providers",
 			DBUS_TYPE_OBJECT_PATH, __connman_provider_list, NULL);
+	connman_dbus_dict_append_array(&dict, "Technologies",
+			DBUS_TYPE_OBJECT_PATH, __connman_technology_list, NULL);
 
-	connman_dbus_dict_append_array(&dict, "Devices",
-			DBUS_TYPE_OBJECT_PATH, __connman_device_list, NULL);
-
-	if (__connman_element_count(NULL, CONNMAN_ELEMENT_TYPE_CONNECTION) > 0)
-		str = "online";
-	else
-		str = "offline";
-
+	str = __connman_notifier_get_state();
 	connman_dbus_dict_append_basic(&dict, "State",
 						DBUS_TYPE_STRING, &str);
 
@@ -154,10 +149,7 @@ static DBusMessage *get_state(DBusConnection *conn,
 					CONNMAN_SECURITY_PRIVILEGE_PUBLIC) < 0)
 		return __connman_error_permission_denied(msg);
 
-	if (__connman_element_count(NULL, CONNMAN_ELEMENT_TYPE_CONNECTION) > 0)
-		str = "online";
-	else
-		str = "offline";
+	str = __connman_notifier_get_state();
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_STRING, &str,
 						DBUS_TYPE_INVALID);
@@ -504,6 +496,48 @@ static DBusMessage *unregister_agent(DBusConnection *conn,
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
+static DBusMessage *register_counter(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	const char *sender, *path;
+	unsigned int interval;
+	int err;
+
+	DBG("conn %p", conn);
+
+	sender = dbus_message_get_sender(msg);
+
+	dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
+						DBUS_TYPE_UINT32, &interval,
+							DBUS_TYPE_INVALID);
+
+	err = __connman_counter_register(sender, path, interval);
+	if (err < 0)
+		return __connman_error_failed(msg, -err);
+
+	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+}
+
+static DBusMessage *unregister_counter(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	const char *sender, *path;
+	int err;
+
+	DBG("conn %p", conn);
+
+	sender = dbus_message_get_sender(msg);
+
+	dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
+							DBUS_TYPE_INVALID);
+
+	err = __connman_counter_unregister(sender, path);
+	if (err < 0)
+		return __connman_error_failed(msg, -err);
+
+	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+}
+
 static GDBusMethodTable manager_methods[] = {
 	{ "GetProperties",     "",      "a{sv}", get_properties     },
 	{ "SetProperty",       "sv",    "",      set_property       },
@@ -522,6 +556,8 @@ static GDBusMethodTable manager_methods[] = {
 						G_DBUS_METHOD_FLAG_ASYNC },
 	{ "RegisterAgent",     "o",     "",      register_agent     },
 	{ "UnregisterAgent",   "o",     "",      unregister_agent   },
+	{ "RegisterCounter",   "ou",    "",      register_counter   },
+	{ "UnregisterCounter", "o",     "",      unregister_counter },
 	{ },
 };
 
@@ -583,7 +619,7 @@ static DBusMessage *nm_state(DBusConnection *conn,
 	if (reply == NULL)
 		return NULL;
 
-	if (__connman_element_count(NULL, CONNMAN_ELEMENT_TYPE_CONNECTION) > 0)
+	if (__connman_notifier_count_connected() > 0)
 		state = NM_STATE_CONNECTED;
 	else
 		state = NM_STATE_DISCONNECTED;
